# -*- coding: utf-8 -*-
from .baseapi import BaseAPI

"""
@author VoiceAI
@desc 本模块是Group API相关
@date 20180928
"""


class AppGroupAPI(BaseAPI):
    def __init__(self, base_url, app_id, access_token):
        super(AppGroupAPI, self).__init__(base_url, app_id)
        self._access_token = access_token
        self._file_id_list = []

    def app_group_create(self, group_name, describe):
        """
        创建用户组
        :param group_name:  用户组名字 string
        :param describe:    用户组描述 string
        :return:
            用户组ID string
        """
        req_json = {
            "access_token": self._access_token,
            "group_name": group_name,
            "describe": describe
        }
        api_response = self.call_common_api("/api/app/group/create", req_json)
        err = api_response["error"]

        group_id = 0
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
            group_id = data["groupid"]

        return flag_bool_response, err, group_id

    def app_group_get(self, group_id="", group_name=""):
        """
        获取用户组
        :param group_id:    用户组ID
        :return:
            响应数据 object
        """
        req_json = {
            "access_token": self._access_token,
            "groupid": group_id,
            "group_name": group_name
        }
        api_response = self.call_common_api("/api/app/group/get", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
        return flag_bool_response, err, data

    def app_group_delete(self, group_id_list):
        """
        删除用户组
        :param group_id_list:   用户组ID列表
        :return:
            响应数据 object
        """
        group_list = []
        for i in range(len(group_id_list)):
            group_list.append({
                "groupid": group_id_list[i]
            })

        req_json = {
            "access_token": self._access_token,
            "group_list": group_list
        }
        api_response = self.call_common_api("/api/app/group/delete", req_json)
        err = api_response["error"]
        data = None

        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
        return flag_bool_response, err, data

    def app_group_count_client(self, group_id, group_name):
        """
        获取用户组下用户数
        :param group_id:   用户组ID string
        :param group_name:   用户组名字 string
        :return:
            响应数据 object
        """
        req_json = {
            "access_token": self._access_token,
            "group_id": group_id,
            "group_name": group_name
        }
        api_response = self.call_common_api("/api/app/group/clientcount", req_json)
        err = api_response["error"]
        data = None

        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
        return flag_bool_response, err, data



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals, division, print_function
import json, math
import logging, time
import threading

# import copy

from ws4py.client.threadedclient import WebSocketBaseClient
from ws4py.messaging import *
import sys

if sys.version_info[0] == 3:
    import queue
else:
    import Queue

MODEL_ASR_POWER = "model_asr_power"
MODEL_ASR_NUMBER = "model_asr_number"

"""
    ASR结束时会异常，请捕获！！
"""


class ASRException(Exception):
    pass


class ASRCloseException(Exception):
    pass


# 建议不使用该类！！！
class ASRBaseClient(WebSocketBaseClient):
    def __init__(self, url, app_id, token, model_type, sample_rate, file_name=None, long_data=None, client_id="",
                 protocols=['http-only', 'chat'], extensions=None, heartbeat_freq=None,
                 ssl_options=None, headers=None, exclude_headers=None, func=None, delay=800):
        """

        :param url:         WS地址
        :param app_id:      APP应用标志
        :param token:       APP授权Token
        :param model_type:  算法模型类型
        :param sample_rate: 采样率
        :param file_name    验证音频文件，优先级低于下述
        :param long_data:   验证音频数据
        :param client_id:
        :param protocols:
        :param extensions:
        :param heartbeat_freq:
        :param ssl_options:
        :param headers:
        :param exclude_headers:
        :param func:            接收服务器文本辅助回调函数（内部使用)
        """
        WebSocketBaseClient.__init__(self, url, protocols, extensions, heartbeat_freq,
                                     ssl_options, headers=headers, exclude_headers=exclude_headers)
        self._th = threading.Thread(target=self.run, name='WebSocketClientDIY')
        self._th.daemon = True

        if sys.version_info[0] == 3:
            self._buffer = queue.Queue()
        else:
            self._buffer = Queue.Queue()
        self._is_closed = False

        self.app_id = app_id
        self.token = token
        self.client_id = client_id
        self.model_type = model_type
        self.sample_rate = sample_rate

        # 800 ms
        if delay <= 0:
            self.delay = 0
        else:
            self.delay = delay / 1000.0
        if self.sample_rate == 8000:
            self.send_one = 16000
        elif self.sample_rate == 16000:
            self.send_one = 32000
        else:
            raise ASRException("parm sample_rate must 8000 or 16000")
        self._is_done = False
        self.long_data = long_data

        self.save = None
        self.func = func
        self.file_name = file_name
        self._is_open = False

    # 不需要了
    # def __del__(self):
    #     if self._is_closed is False:
    #         self.close()

    @property
    def daemon(self):
        return self._th.daemon

    @daemon.setter
    def daemon(self, flag):
        self._th.daemon = flag

    def run_forever(self):
        while not self.terminated:
            self._th.join(timeout=0.1)

    def handshake_ok(self):
        self._th.start()

    def closed(self, code, reason=None):
        logging.debug("websocket close:%r" % reason)
        self._is_closed = True

    def opened(self):
        self._is_closed = False
        d = {
            "appid": self.app_id,
            "access_token": self.token,
            "clientid": self.client_id,
            "sample_rate": self.sample_rate,
            "model_type": self.model_type

        }

        o = json.dumps(d)
        logging.debug("websocket client open")
        logging.debug("auth: %s" % o)
        self.send(o)
        self._is_open = True

        th1 = threading.Thread(target=ASRBaseClient.handle, args=(self,))
        th1.start()
        # th1.join()

    # 不要直接用
    def handle(self):
        while True:
            if self._is_open is True:
                # 二进制优先
                if self.long_data is not None:
                    temp = self.long_data
                    len_wav_data = len(temp)
                    if len(temp) > self.send_one:
                        times = int(math.ceil(len_wav_data / float(self.send_one)))
                        for i in range(0, times):
                            start = i * self.send_one
                            end = (i + 1) * self.send_one
                            # last
                            if i == times - 1:
                                raw = temp[start:len_wav_data + 1]
                            else:
                                raw = temp[start:end]
                            self.send(self._wrap_head(raw), True)
                            if self.delay > 0:
                                time.sleep(self.delay)
                    else:
                        self.send(self._wrap_head(self.long_data), True)

                    self.send(self._wrap_head(None, True), True)
                elif self.file_name is not None:
                    fd = open(self.file_name, "rb")
                    size = self.send_one
                    times = 1
                    while True:
                        raw = fd.read(size)
                        if len(raw) == 0:
                            self.send(self._wrap_head(None, True), True)
                            break
                        fd.seek(times * size)
                        times = times + 1
                        try:
                            self.send(self._wrap_head(raw), True)
                        except:
                            raise ASRCloseException("server sudden close the connection")
                        # logging.debug("sleep :%f" % self.delay)
                        if self.delay > 0:
                            time.sleep(self.delay)
                return

    def _wrap_head(self, data, final=False):
        if data is not None:
            raw = bytearray(data)
            length = len(raw)
        else:
            length = 0
        header = bytearray(32)
        header[0:4] = b'CRPV'
        header[4] = 1
        header[5] = 0
        header[6] = 0
        header[7] = 0
        header[8] = 1
        header[9] = 0
        header[10] = 1
        if final is True:
            header[11] = 1
            header[24] = 1
        else:
            header[11] = 0
            header[24] = 0

        byte_rate = length + 16
        header[12] = byte_rate & 0xff
        header[13] = (byte_rate >> 8) & 0xff
        header[14] = (byte_rate >> 16) & 0xff
        header[15] = (byte_rate >> 24) & 0xff
        header[16:20] = b'CRPV'
        header[20] = 16
        header[21] = 0
        header[22] = 0
        header[23] = 0
        header[25] = 0
        header[26] = 0
        header[27] = 0
        header[28] = 0
        header[29] = 0
        header[30] = 0
        header[31] = 0

        if data is not None:
            header.extend(data)
        logging.debug("header go: %r" % header)
        return header

    def _reverse(self, hex_raw):
        raw_len = len(hex_raw)
        if raw_len <= 43:
            return None

        re = list()
        i = 0

        # for ii in range(0, raw_len):
        #     print("%d： %#x：%s" % (ii, hex_raw[ii], chr(hex_raw[ii])))
        # exit(1)
        # print("================")
        while True:
            if i >= raw_len:
                break
            if hex_raw[i:i + 4] == b'CRPV':
                star = i
                u8a_number = bytearray(hex_raw[star + 22:star + 26])
                numb = ((u8a_number[0] & 0xFF) | ((u8a_number[1] & 0xFF) << 8) | ((u8a_number[2] & 0xFF) << 16) | (
                        (u8a_number[3] & 0xFF) << 24))
                end = star + 43 + numb
                dest = bytearray(hex_raw[star: end])
                re.append(dest)
                i = end - 1
            i = i + 1

        for i in re:
            u8a_offset = bytearray(i[18:22])

            offset = ((u8a_offset[0] & 0xFF) |
                      ((u8a_offset[1] & 0xFF) << 8) |
                      ((u8a_offset[2] & 0xFF) << 16) |
                      ((u8a_offset[3] & 0xFF) << 24))

            u8a_number = bytearray(i[22:26])
            numb = ((u8a_number[0] & 0xFF) |
                    ((u8a_number[1] & 0xFF) << 8) |
                    ((u8a_number[2] & 0xFF) << 16) |
                    ((u8a_number[3] & 0xFF) << 24))

            isfinal = i[26]

            # 关闭他！！
            if isfinal == 1:
                self._is_done = True
                self.close()

            # print("offset: %d number: %d isfinal: %d" % (offset, numb, isfinal))
            logging.debug("offset: %d number: %d isfinal: %d" % (offset, numb, isfinal))

            sss = bytearray(i[43:])

            if numb > 0:
                if self.save is not None and offset > 0 and offset < len(self.save):
                    self.save[offset:offset + len(sss) + 1] = sss
                else:
                    self.save = bytearray(sss)

        if self.save is not None:
            data = self.save
            return data.decode("utf-8", "ignore")

    def received_message(self, m):
        text = None
        if isinstance(m, BinaryMessage):
            hex_raw = bytearray(m.data)
            logging.debug("receive binary: %r" % hex_raw)

            text = self._reverse(hex_raw)
        elif isinstance(m, TextMessage):
            logging.debug("receive text: %r" % m.data)
            try:
                text = json.loads(str(m))
            except Exception as e:
                raise ASRException(repr(e))

        else:
            return
            # print(m)

        if text is not None:
            logging.debug("put buffer %s" % text)
            self._buffer.put(text)

        if self.save is not None and len(self.save) > 0:
            if self.func is not None:
                self.func(self.save)

                # if len(self.save) > 100:
                #     self.save = bytearray()
                return

    ################## 以下为可用方法，以上方法不要用！！ #################

    # 获取数据
    def get_real_time_txt(self):
        if self._buffer.empty() is False:
            result = self._buffer.get_nowait()

            # logging.debug(result)

            # python2/python3
            if type(result) == type(""):
                return True, result

            # {"flag":true,"error":null,"data":{"type":0}}
            # {"flag":false,"error":{"errorid":"10000","errormsg":"内部错误(算法模型不存在)"},"data":null}
            if result is not None and "error" in result and result["error"] is not None:
                # if self._is_closed is False:
                # self.close()
                raise Exception("%s-%s" % (result["error"]["errormsg"], result["error"]["errorid"]))
            return True, None

        if self._is_closed:
            if self._buffer.empty() is False:
                return False, None
            else:
                raise ASRCloseException("websocket client close")
        return False, None

    # 获取最终结果
    def get_final_txt(self):
        if self.save is not None:
            return self.save.decode("utf-8", "ignore")
        return ASRException("result empty, check the model and rate")

    # 自定发送二进制： 请不要用！！
    def diy_send_binary(self, data):
        if self._is_closed:
            raise ASRCloseException("websocket client close")
        if self.long_data is not None or self.file_name is not None:
            raise ASRException("don't do this action diy_send_binary")
        self.send(self._wrap_head(data), True)

    # 自定发送二进制结束： 请不要用！！
    def diy_send_binary_end(self):
        if self.long_data is not None or self.file_name is not None:
            raise ASRException("don't do this action diy_send_binary_end")
        self.send(self._wrap_head(None, True), True)



# Public Domain, i.e. feel free to copy/paste
# Considered a hack in Python 2
import inspect
import logging
import sys


def mylog(skip=0):
    '''Shorthand to get logger for current function frame.'''
    return logging.getLogger(caller_name(skip + 1))


def caller_name(skip=2):
    """Get a name of a caller in the format module.class.method

       `skip` specifies how many levels of stack to skip while getting caller
       name. skip=1 means "who calls me", skip=2 "who calls my caller" etc.

       An empty string is returned if skipped levels exceed stack height
    """

    def stack_(frame):
        framelist = []
        while frame:
            framelist.append(frame)
            frame = frame.f_back
        return framelist

    stack = stack_(sys._getframe(1))
    start = 0 + skip
    if len(stack) < start + 1:
        return ''
    parentframe = stack[start]

    name = []
    module = inspect.getmodule(parentframe)
    # `modname` can be None when frame is executed directly in console
    # TODO(techtonik): consider using __main__
    if module:
        name.append(module.__name__)
    # detect classname
    if 'self' in parentframe.f_locals:
        # I don't know any way to detect call from the object method
        # XXX: there seems to be no way to detect static method call - it will
        #      be just a function call
        name.append(parentframe.f_locals['self'].__class__.__name__)
    codename = parentframe.f_code.co_name
    if codename != '<module>':  # top level usually
        name.append(codename)  # function or a method
    del parentframe
    return ".".join(name)



# -*- coding: utf-8 -*-
"""
    pyvoiceai public
"""

from .app_voiceprint_api import AppVoicePrintAPI
from .app_client_api import AppClientAPI
from .app_group_api import AppGroupAPI
from .app_auth_api import AppAuthAPI
from .baseapi import BaseAPI
from .log import mylog
from .app_asr_base_api import *
from .app_asr import *

# Set default logging handler to avoid "No handler found" warnings.
import logging

try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())



# -*- coding: utf-8 -*-
from .baseapi import BaseAPI

"""
@author VoiceAI
@desc 本模块是Client API相关
@date 20180928
"""


class AppClientAPI(BaseAPI):
    def __init__(self, base_url, app_id, access_token):
        super(AppClientAPI, self).__init__(base_url, app_id)
        self._access_token = access_token
        self._file_id_list = []

    def app_client_create(self, client_name, describe, group_id, group_name=""):
        """
        创建用户
        :param client_name: 用户名 string
        :param describe:    用户描述 string
        :param group_id:    用户组ID string
        :return:
                成功标志 bool
                用户ID string
        """
        req_json = {
            "access_token": self._access_token,
            "client_name": client_name,
            "describe": describe,
            "groupid": group_id,
            "group_name": group_name
        }
        api_response = self.call_common_api("/api/app/client/create", req_json)
        err = api_response["error"]
        client_id = ""
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
            client_id = data["clientid"]
        return flag_bool_response, err, client_id

    def app_client_get(self, group_id, group_name="", client_id="", client_name="", page=0, limit=100):
        """
        获取用户
        :param group_id:    用户组ID string
        :param client_id:   用户ID string option
        :return:
            响应数据 object
        """
        req_json = {
            "access_token": self._access_token,
            "groupid": group_id,
            "clientid": client_id,
            "client_name": client_name,
            "group_name": group_name,
            "page": page,
            "limit": limit
        }
        api_response = self.call_common_api("/api/app/client/get", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]

        return flag_bool_response, err, data

    def app_client_delete(self, client_id_list, groupid="", group_name=""):
        """
        删除用户
        :param client_id_list: 用户ID列表 list
        :return:
            响应数据 object
        """
        client_list = []
        for i in range(len(client_id_list)):
            client_list.append({
                "clientid": client_id_list[i]
            })

        req_json = {
            "access_token": self._access_token,
            "client_list": client_list,
            "groupid": groupid,
            "group_name": group_name,
        }
        api_response = self.call_common_api("/api/app/client/delete", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]

        return flag_bool_response, err, data



# -*- coding: utf-8 -*-

from Cryptodome.Hash import SHA256

from .baseapi import BaseAPI

"""
@author VoiceAI
@desc 本模块是AuthAPI认证
@date 20180928
"""


class AppAuthAPI(BaseAPI):
    def __init__(self, base_url, app_id, access_token=""):
        """

        :param base_url:        声纹云URL地址 string
        :param app_id:          应用ID string
        :param access_token:    访问Token string option
        """
        super(AppAuthAPI, self).__init__(base_url, app_id)
        self._app_secret = ""
        self._nonce = ""
        self._timestamp = 0
        self._access_token = access_token
        self._expires = 7200

    def get_access_token(self):
        """

        :return: 返回访问Token string
        """
        return self._access_token

    def get_expires(self):
        """

        :return: 返回访问Token过期时间 int
        """
        return self._expires

    def app_auth_get(self, app_secret):
        """
        获取Nonce等方便签名获取Token
        :param app_secret:  应用密钥 string
        :return:
            响应数据 object
        """
        self._app_secret = app_secret
        api_response = self.call_common_api("/api/app/auth/get", {})
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
            self._nonce = data["nonce"]
            self._timestamp = data["timestamp"]

        return flag_bool_response, err, data

    def app_auth_token_get(self):
        """
        获取Token
        :return:
            响应数据 object
        """
        content = [self._app_id, self._app_secret, self._nonce, str(self._timestamp)]
        h = SHA256.new()
        content.sort()
        h.update("".join(content).encode("utf8"))
        signature = h.hexdigest()
        req_json = {
            "nonce": self._nonce,
            "timestamp": self._timestamp,
            "signature": signature
        }
        api_response = self.call_common_api("/api/app/auth/token/get", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
            self._access_token = data["access_token"]
            self._expires = data["expires"]

        return flag_bool_response, err, data

    def app_auth_token_refresh(self):
        """
        刷新Token
        :return:
            响应数据 object
        """
        req_json = {
            "access_token": self._access_token
        }
        api_response = self.call_common_api("/api/app/auth/token/refresh", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
            self._expires = data["expires"]

        return flag_bool_response, err, data

    def app_auth_token_remove(self):
        """
        移除Token
        :return:
            响应数据 object
        """
        req_json = {
            "access_token": self._access_token
        }
        api_response = self.call_common_api("/api/app/auth/token/remove", req_json)
        err = api_response["error"]
        flag_bool_response = bool(api_response["flag"])
        return flag_bool_response, err



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals, division, print_function
import datetime
import json
import logging

import requests

# 禁用安全请求警告
try:
    requests.packages.urllib3.disable_warnings()
except:
    pass

"""
@author VoiceAI
@desc 本模块是API基础类
@date 20180928
"""


class BaseAPI(object):
    def __init__(self, base_url, app_id):
        self._base_url = base_url
        self._app_id = app_id
        self.logCallback = callable

    def call_common_api(self, api_path, req_data):
        """
        一般API请求基本接口
        :param api_path:    请求API的URL string
        :param req_data:    请求API的JSON数据 dict
        :return:
            JSON响应数据 object
        """
        logging.debug("call_common_api:%s" % (self._base_url + api_path))
        time_start = datetime.datetime.now()

        full_url = self._base_url + api_path
        base_data = {
            "appid": self._app_id
        }
        final_data = dict(base_data, **req_data)
        logging.debug("request:%s", json.dumps(final_data))
        try:
            r = requests.post(full_url, json=final_data, verify=False)
            logging.debug("response:%r" % r.content)
            rsp_data = r.json()
        except Exception as e:
            rsp_data = {"flag": False, "error": {"errorid": "99999", "errormsg": str(e)}}

        time_end = datetime.datetime.now()
        cost_time = (time_end - time_start).microseconds
        logging.debug("call_common_api 耗时：%f seconds" % (float(cost_time) / 1000000))
        return rsp_data

    def call_upload_api(self, api_path, req_data, files):
        """
        上传文件接口
        :param api_path:    请求API的URL string
        :param req_data:    请求API的JSON数据 dict
        :param files:       上传的文件路径 string
        :return:
            JSON响应数据 object
        """
        logging.debug("call_upload_api:%s" % (self._base_url + api_path))
        time_start = datetime.datetime.now()
        full_url = self._base_url + api_path
        base_data = {
            "appid": self._app_id
        }
        final_data = dict(base_data, **req_data)

        try:
            r = requests.post(full_url, data=final_data, files=files, verify=False)
            logging.debug("response:%r" % r.content)
            rsp_data = r.json()
        except Exception as e:
            rsp_data = {"flag": False, "error": {"errorid": "99999", "errormsg": str(e)}}

        time_end = datetime.datetime.now()
        cost_time = (time_end - time_start).microseconds
        logging.debug("call_upload_api 耗时：%f seconds" % (float(cost_time) / 1000000))
        return rsp_data

    def call_download_dia_result_api(self, api_path, token, file_id):
        """
        临时辅助下载人声分割结果文件
        :param api_path:
        :param file_id:
        :return:
        """
        logging.debug("call_download_dia_result_api:%s" % (self._base_url + api_path))
        time_start = datetime.datetime.now()
        full_url = self._base_url + api_path
        flag = False
        try:
            r = requests.get(full_url, headers={"appid": self._app_id, "access_token": token, "fileid": file_id},
                             verify=False)
            flag = True
            raw_data = r.content
        except Exception as e:
            err = {"flag": False, "error": {"errorid": "99999", "errormsg": str(e)}}

        time_end = datetime.datetime.now()
        cost_time = (time_end - time_start).microseconds
        logging.debug("call_download_dia_result_api 耗时：%f seconds" % (float(cost_time) / 1000000))
        if flag:
            return flag, raw_data
        return flag, err



# -*- coding: utf-8 -*-
import os
import sys

from setuptools import setup

description = "VoiceaiTech Python SDK"
about = open("./Log.txt", "rb").read().decode("utf-8", "ignore")
# print(about)

Version = '1.3.8'


def git_push():
    os.system('git add --all')
    os.system('git commit -m \"setup python dodo:v%s\"' % Version)
    os.system("git tag -d v%s" % Version)
    os.system("git tag v%s" % Version)
    os.system("git push origin :refs/tags/v%s" % Version)
    os.system("git push")
    os.system("git push --tags")


# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    git_push()
    # pip install wheel
    os.system("rm -rf dist/*")
    os.system('python setup.py sdist bdist_wheel')
    # pip install twine
    os.system('twine upload dist/* --verbose')
    sys.exit()

if sys.argv[-1] == 'test':
    os.system('python setup.py develop')
    sys.exit()

if sys.argv[-1] == 'git':
    git_push()
    sys.exit()

packages = ['pyvoiceai']

requires = [
    'requests>=2.19.1',
    'pycryptodomex>=3.6.6',
    "pyOpenSSL>=18.0.0",
    "ws4py>=0.5.1"
]

setup(
    name='pyvoiceai',  # 应用名
    version=Version,  # 版本号
    author="voiceai",
    author_email="voiceai-github@voiceaitech.com",
    url="https://pypi.org/project/pyvoiceai/",
    description=description,
    long_description=about,
    license="",
    packages=packages,
    package_dir={packages[0]: 'pyvoiceai'},
    platforms=['any'],
    install_requires=requires,
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
)



# -*- coding: utf-8 -*-

from .baseapi import BaseAPI

"""
@author VoiceAI
@desc 本模块是VPR API相关
@date 20180928
"""


class AppVoicePrintAPI(BaseAPI):
    def __init__(self, base_url, app_id, access_token):
        super(AppVoicePrintAPI, self).__init__(base_url, app_id)
        self._access_token = access_token
        self._file_id_list = []

    def get_file_id_list(self):
        """
        获取文件ID列表
        :return: 上次上传文件后的文件标志ID string
        """
        return self._file_id_list

    def app_voiceprint_upload(self, file_path_list, media_type="", sample_rate=""):
        """
        音频文件上传
        :param client_id:       用户ID string
        :param file_path_list:  文件位置列表 list
        :return:
            响应数据 object
        """
        self._file_id_list = []
        req_json = {
            "access_token": self._access_token,
            "media_type": media_type,
            "sample_rate": sample_rate
        }
        files = {}
        for i in range(len(file_path_list)):
            files["file%s" % str(i)] = ("file%s" % str(i), open(file_path_list[i], "rb"))

        api_response = self.call_upload_api("/api/app/voiceprint/upload", req_json, files)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
            file_info_list = data["list"]
            for i in range(len(file_info_list)):
                file_info = file_info_list[i]
                self._file_id_list.append(file_info["fileid"])

        return flag_bool_response, err, data

    def app_voiceprint_register(self, group_id, client_id, src_sample_rate, model_type, file_id_list,
                                group_name="", client_name="", debug=False, debug_data=[]):
        """
        声纹注册
        :param client_id:       客户ID string
        :param group_id:        组ID string
        :param client_name:     客户名字 string  client_id二选一
        :param group_name:      组名字 string   group_id二选一
        :param src_sample_rate: 采样率 int
        :param model_type:      模型类型 string
        :param file_id_list:    文件ID列表 list
        :return:
            响应数据 object
        """
        req_json = {
            "clientid": client_id,
            "src_sample_rate": src_sample_rate,
            "model_type": model_type,
            "access_token": self._access_token,
            "filelist": file_id_list,
            "groupid": group_id,
            "client_name": client_name,
            "group_name": group_name,
            "no_save_file": debug,
            "no_save_file_data": debug_data
        }
        api_response = self.call_common_api("/api/app/voiceprint/register", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]
        return flag_bool_response, err, data

    def app_voiceprint_verify(self, group_id, client_id, src_sample_rate, model_type, file_id_list, get_all_data=True,
                              group_name="", client_name=""):
        """
        声纹验证
        :param group_id:    用户组ID string
        :param client_id:   客户ID string,留空表示1:N
        :param group_name:      组名字 string   group_id二选一
        :param client_name:     客户名字 string  client_id二选一 留空表示1:N
        :param src_sample_rate: 采样率 int
        :param model_type:      模型类型 string
        :param file_id_list:    文件ID列表 list
        :param get_all_data:    是否获取所有信息 bool option
        :return:
            响应数据 object
        """
        req_json = {
            "clientid": client_id,
            "groupid": group_id,
            "src_sample_rate": src_sample_rate,
            "model_type": model_type,
            "access_token": self._access_token,
            "filelist": file_id_list,
            "get_all_data": get_all_data,
            "client_name": client_name,
            "group_name": group_name
        }
        api_response = self.call_common_api("/api/app/voiceprint/verify", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]

        return flag_bool_response, err, data

    def app_voiceprint_asvcheck(self, src_sample_rate, model_type, file_id_list, asv_threshold=None):
        """
        活体检测
        :param src_sample_rate: 采样率 int
        :param model_type:      模型类型 string
        :param file_id_list:    文件ID列表 list
        :param asv_threshold:   unknown
        :return:
            响应数据 object
        """
        req_json = {
            "src_sample_rate": src_sample_rate,
            "model_type": model_type,
            "access_token": self._access_token,
            "filelist": file_id_list,
            "asv_threshold": asv_threshold
        }
        api_response = self.call_common_api("/api/app/voiceprint/asvcheck", req_json)
        err = api_response["error"]
        data = None

        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]

        return flag_bool_response, err, data

    def app_voiceprint_dialaunch(self, group_id, client_id, src_sample_rate, model_type, file_id,
                                 describe="empty des", num_of_speakers_specified=0,
                                 group_name="", client_name=""):
        """
        人声分割
        :param group_id:        用户组ID string
        :param client_id:       客户ID string
        :param client_name:     客户名字 string  client_id二选一
        :param group_name:      组名字 string   group_id二选一
        :param src_sample_rate: 采样率 int
        :param model_type:      模型类型 string
        :param file_id:         文件ID string
        :param describe         描述 string
        :param num_of_speakers_specified unknown optional
        :return:
            响应数据 object
        """
        req_json = {
            "src_sample_rate": src_sample_rate,
            "model_type": model_type,
            "access_token": self._access_token,
            "fileid": file_id,
            "clientid": client_id,
            "groupid": group_id,
            "num_of_speakers_specified": num_of_speakers_specified,
            "describe": describe,
            "client_name": client_name,
            "group_name": group_name
        }
        api_response = self.call_common_api("/api/app/voiceprint/dia", req_json)
        err = api_response["error"]
        task_id = ""
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            task_id = api_response["data"]["task_id"]

        return flag_bool_response, err, task_id

    def app_voiceprint_dia_task_list(self, group_id, client_id, page=0, limit=10, group_name="", client_name=""):
        req_json = {
            "access_token": self._access_token,
            "clientid": client_id,
            "groupid": group_id,
            "page": page,
            "limit": limit,
            "client_name": client_name,
            "group_name": group_name
        }
        api_response = self.call_common_api("/api/app/voiceprint/dia/task", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]

        return flag_bool_response, err, data

    def app_voiceprint_dia_task_list_one(self, group_id, client_id, task_id):
        req_json = {
            "access_token": self._access_token,
            "clientid": client_id,
            "groupid": group_id,
            "task_id": task_id
        }
        api_response = self.call_common_api("/api/app/voiceprint/dia/result", req_json)
        err = api_response["error"]
        data = None
        flag_bool_response = bool(api_response["flag"])
        if not flag_bool_response:
            pass
        else:
            data = api_response["data"]

        return flag_bool_response, err, data

    def app_voiceprint_dia_result_download(self, file_id):
        return self.call_download_dia_result_api("/api/app/voiceprint/dia/download", self._access_token, file_id)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals, division, print_function
from .app_auth_api import *
from .app_asr_base_api import *

MODEL_ASR_POWER = "model_asr_power"
MODEL_ASR_NUMBER = "model_asr_number"


class ASRClient:
    def __init__(self, app_id, app_secret, host="wss://127.0.0.1:8072", call_back=None):
        self.socket_host = host
        if self.socket_host.endswith("/"):
            self.socket_host = self.socket_host + "api/app/asr/streaming"
        else:
            self.socket_host = self.socket_host + "/api/app/asr/streaming"
        url = ""
        if "wss://" in host:
            url = host.replace("wss://", "https://")
        elif "ws://" in host:
            url = host.replace("ws://", "http://")
        else:
            raise ASRException("host must prefix wss:// or ws://")
        self.auth = AppAuthAPI(app_id=app_id, base_url=url)
        ok, err, data = self.auth.app_auth_get(app_secret)
        if ok is False:
            raise ASRException(data)
        ok1, err, data1 = self.auth.app_auth_token_get()
        if ok1 is False:
            raise ASRException(data1)
        self.auth.app_auth_token_refresh()
        self.token = self.auth.get_access_token()
        self.app_id = app_id
        self.call_back = call_back

    def asr(self, sample_rate, model_type, file_name, delay=800):
        self.auth.app_auth_token_refresh()
        ws = ASRBaseClient(url=self.socket_host, app_id=self.app_id, token=self.token, sample_rate=sample_rate,
                           model_type=model_type, file_name=file_name, delay=delay)
        ws.daemon = True
        try:
            ws.connect()
            logging.debug("wait, please...may be long long.")
            while True:
                ok, result = ws.get_real_time_txt()
                if ok and result is not None:
                    if self.call_back is not None:
                        self.call_back(result)
                        # logging.debug("asr2:%s" % result)
        except ASRCloseException as e:
            # 获取最终结果
            logging.debug("asr err inner %r" % e)
            result = ws.get_final_txt()
            return result

        return ""
